//
// Created by nicolas on 23/03/18.
//

#include "event.h"
#include "debug.h"

#ifdef _WIN32
#include <windows.h>

struct st_event {
    HANDLE handle;
};

event_t* event_create() {
    event_t* event = malloc(sizeof(struct st_event));
    event->handle = CreateEvent(NULL, FALSE, FALSE, NULL);
    return event;
}

void event_destroy(event_t* event) {
    CloseHandle(event->handle);
    free(event);
}

void event_wait(event_t* event) {
    WaitForSingleObject(event->handle, INFINITE);
}

void event_set(event_t* event) {
    SetEvent(event->handle);
}

#endif

#ifdef __linux__
#include <pthread.h>

struct st_event {
    pthread_mutex_t mutex;
    pthread_cond_t handle;
};

event_t* event_create() {
    event_t* event = malloc(sizeof(event_t));
    pthread_mutex_init(&event->mutex, 0);
    pthread_cond_init(&event->handle, 0);
    return event;
}

void event_destroy(event_t* event) {
    pthread_mutex_destroy(&event->mutex);
    pthread_cond_destroy(&event->handle);
    free(event);
}

void event_wait(event_t* event) {
    pthread_cond_wait(&event->handle, &event->mutex);
}

void event_set(event_t* event) {
    pthread_cond_broadcast(&event->handle);
}

#endif